﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using INIReader;
using System.IO;
namespace KeyKeeper
{
    public partial class frmmain : Form
    {

        string Filename = "";
        string SelectItem = "";
        int SelectIndex = 0;

        private string GetInfo()
        {
            //Create ini object
            cINI ini = new cINI(Filename);
            //Get info
            return ini.GetValue(SelectItem, "info", "");
        }

        private void Export(string iFilename)
        {
            StringBuilder sb = new StringBuilder();
            //Loop tho the items in list view and build a list
            foreach (ListViewItem li in LstV.Items)
            {
                sb.Append("Title:      " + li.Text + Environment.NewLine);
                sb.Append("Serial:     " + li.SubItems[1].Text + Environment.NewLine);
                sb.Append(Environment.NewLine);
            }

            try
            {
                using (StreamWriter sw = new StreamWriter(iFilename))
                {
                    sw.Write(sb.ToString());
                    sw.Close();
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private int DeleteKey()
        {
            string iCaption = "";
            string iKey = "";
            string iInfo = "";
            int counter = 1;
            int iCounter = 0;
            StringBuilder sb = new StringBuilder();
            //Create ini object
            cINI ini = new cINI(Filename);
            //Get item count
            int Items = Convert.ToInt32(ini.GetValue("Main", "Items", "0"));
            //Build ini header
            sb.Append("[main]" + Environment.NewLine);
            sb.Append("items=" + (Items - 1) + Environment.NewLine);
            sb.Append(Environment.NewLine);

            //Loop tho all the items
            for (counter = 1; counter <= Items; counter++)
            {
                //Get item data
                iCaption = ini.GetValue("Item" + counter, "Caption", "Untitled");
                iKey = ini.GetValue("Item" + counter, "Key", "");
                iInfo = ini.GetValue("Item" + counter, "Info", "");
                //
                if (counter != SelectIndex + 1)
                {
                    iCounter++;
                    //append to ini file
                    sb.Append("[item" + iCounter + "]" + Environment.NewLine);
                    sb.Append("caption=" + iCaption + Environment.NewLine);
                    sb.Append("key=" + iKey + Environment.NewLine);
                    sb.Append("info=" + iInfo + Environment.NewLine);
                    sb.Append(Environment.NewLine);
                }
            }
            try
            {
                //Write over the ini file
                using (StreamWriter sw = new StreamWriter(Filename))
                {
                    //Write string builder to file
                    sw.Write(sb.ToString());
                    sw.Close();
                    return 1;
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return 0;
            }
        }

        private int EditKey()
        {
            //Check if file is readonly
            FileInfo fi = new FileInfo(Filename);

            if (fi.IsReadOnly)
            {
                return 0;
            }
            else
            {
                //Create new ini object
                cINI ini = new cINI(Filename);
                //Edit Item
                ini.SetValue(SelectItem, "Caption", Tools.KeyInfo.iCaption);
                ini.SetValue(SelectItem, "Key", Tools.KeyInfo.iKey);
                ini.SetValue(SelectItem, "info", Tools.KeyInfo.iInfo);
                return 1;
            }
        }

        private int AddNewKey()
        {
            //Check if the file is readonly
            FileInfo fi = new FileInfo(Filename);
           
            if (fi.IsReadOnly)
            {
                return 0;
            }

            //Create ini object
            cINI ini = new cINI(Filename);
            //Get number of items
            int Items = Convert.ToInt32(ini.GetValue("Main", "Items", "0")) + 1;
            //Update item count
            ini.SetValue("Main", "Items", Items.ToString());
            //Add new items
            ini.SetValue("Item" + Items, "Caption", Tools.KeyInfo.iCaption);
            ini.SetValue("Item" + Items, "Key", Tools.KeyInfo.iKey);
            ini.SetValue("Item" + Items, "Info", Tools.KeyInfo.iInfo);
            return 1;
        }

        private void LoadKeys(int SelectIndex)
        {
            cINI ini = new cINI(Filename);
            string iCaption = "";
            string iKey = "";
            //Get number of items
            int Items = Convert.ToInt32(ini.GetValue("main", "items", "0"));

            //Clear listview
            LstV.Items.Clear();
            for (int Counter = 1; Counter <= Items; Counter++)
            {
                //Get caption
                iCaption = ini.GetValue("item" + Counter, "caption", "Untitled");
                iKey = ini.GetValue("item" + Counter, "key", "");
                //Add to listview
                LstV.Items.Add(iCaption, 0);
                //Add subitem
                LstV.Items[LstV.Items.Count - 1].SubItems.Add(iKey);
                LstV.Items[LstV.Items.Count - 1].Tag = "item" + Counter;
            }
            //Enable / disable find button
            Button4.Enabled = (LstV.Items.Count > 0);
            Button8.Enabled = (LstV.Items.Count > 0);
            Button9.Enabled = (LstV.Items.Count > 0);

            //Check for items
            if (LstV.Items.Count > 0)
            {
                LstV.Items[SelectIndex].Selected = true;
            }
        }

        public frmmain()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Filename = Path.Combine(Directory.GetCurrentDirectory(),"keys.key");
            string sLine = "";
            string FileExt = "";
            string[] args = Environment.GetCommandLineArgs();
            lblInfo1.Text = "";

            StringBuilder sb = new StringBuilder();
            //Check if keys file is found if not create it

            if (args.Length == 1)
            {

                if (!File.Exists(Filename))
                {
                    //Create file
                    using (StreamWriter sw = new StreamWriter(Filename))
                    {
                        //Build ini header
                        sb.Append("[main]" + Environment.NewLine);
                        sb.Append("item=0" + Environment.NewLine);
                        sb.Append(Environment.NewLine);
                        //Write ini header
                        sw.Write(sb.ToString());
                        sw.Close();
                    }
                }

                //Load the keys
                LoadKeys(0);
                //Raise resize event
                frmmain_Resize(sender, e);
            }

            //Get if opening a file form command line
            if (args.Length == 2)
            {
                sLine = args[1];
                //Get file extension
                FileExt = Path.GetExtension(sLine).ToUpper();
                //Check for vaild extension

                if (FileExt != ".KEY")
                {
                    MessageBox.Show("This is not a vaild Key file", "Error",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                    this.Close();
                }
                else
                {
                    //Load the file
                    Filename = sLine;
                    //Load the leys
                    LoadKeys(0);
                    //Raise resize event
                    frmmain_Resize(sender, e);
                    //Cleat up
                    sLine = string.Empty;
                    FileExt = string.Empty;
                }
            }
        }

        private void Button1_Click(object sender, EventArgs e)
        {
            Tools.KeyAddOp = Tools.AddOp.Add;
            KeyAdd frm = new KeyAdd();
            frm.ShowDialog();

            if (Tools.ButtonPress == 1)
            {
                //Add new key to ini file
                if (AddNewKey() == 0)
                {
                    MessageBox.Show("There was an error adding the item '" + Tools.KeyInfo.iCaption + "'" ,
                        "Add", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    //Reload the items
                    LoadKeys(LstV.Items.Count);
                }
            }
            Tools.ButtonPress = 0;
        }

        private void Button2_Click(object sender, EventArgs e)
        {
            Tools.KeyAddOp = Tools.AddOp.Edit;
            KeyAdd frm = new KeyAdd();
            frm.ShowDialog();

            if (Tools.ButtonPress == 1)
            {
                if (EditKey() == 0)
                {
                    MessageBox.Show("There was an error updating the item '" + Tools.KeyInfo.iCaption + "'",
                        "Edit", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    //Reload the items
                    LoadKeys(SelectIndex);
                }
            }

            Tools.ButtonPress = 0;
        }

        private void LstV_SelectedIndexChanged(object sender, EventArgs e)
        {
            foreach (ListViewItem lv in LstV.SelectedItems)
            {
                //Store tag
                SelectItem = (string)lv.Tag;

                //Store info
                Tools.KeyInfo.iCaption = lv.Text;
                Tools.KeyInfo.iKey = lv.SubItems[1].Text;
                Tools.KeyInfo.iInfo = GetInfo();
                //Display infi
                lblInfo1.Text = Tools.KeyInfo.iInfo;
                //Store selected index
                SelectIndex = lv.Index;
                //Enable buttons
                Button2.Enabled = true;
                Button3.Enabled = true;
                mnuEdit.Enabled = true;
                mnuDelete.Enabled = true;
                mnuCopy.Enabled = true;
            }
        }

        private void LstV_DoubleClick(object sender, EventArgs e)
        {
            Button2_Click(sender, e);
        }

        private void Button3_Click(object sender, EventArgs e)
        {
            DialogResult dr = MessageBox.Show("Are you sure you want to delete the item '" +
               Tools.KeyInfo.iCaption + "'", "Delete", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

            if (dr == DialogResult.Yes)
            {
                //Delete the item from the ini file
                if (DeleteKey() == 1)
                {
                    //Clear info display
                    lblInfo1.Text = "";
                    //Reload the keys
                    LoadKeys(0);
                }

                //Enable / disable buttons / menu items
                Button2.Enabled = (LstV.Items.Count > 0);
                Button3.Enabled = (LstV.Items.Count > 0);
                Button4.Enabled = (LstV.Items.Count > 0);
                mnuEdit.Enabled = (LstV.Items.Count > 0);
                mnuDelete.Enabled = (LstV.Items.Count > 0);
                mnuCopy.Enabled = (LstV.Items.Count > 0);
                
            }
        }

        private void frmmain_Resize(object sender, EventArgs e)
        {
            LstV.Columns[1].Width = -2;
        }

        private void mnuAdd_Click(object sender, EventArgs e)
        {
            Button1_Click(sender, e);
        }

        private void mnuEdit_Click(object sender, EventArgs e)
        {
            Button2_Click(sender, e);
        }

        private void mnuDelete_Click(object sender, EventArgs e)
        {
            Button3_Click(sender, e);
        }

        private void mnuCopy_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(Tools.KeyInfo.iKey))
            {
                Clipboard.SetText(Tools.KeyInfo.iKey, TextDataFormat.Text);
            }
        }

        private void Button4_Click(object sender, EventArgs e)
        {
            Find frm = new Find();
            frm.ShowDialog();
            int FindIndex = -1;
            //
            if (Tools.ButtonPress == 1)
            {
                //Look tho the items in listview
                foreach (ListViewItem li in LstV.Items)
                {
                    if (li.Text.ToLower() == Tools.FindItem.ToLower())
                    {
                        FindIndex = li.Index;
                    }
                }
                //
                if (FindIndex != -1)
                {
                    //Select the index
                    LstV.Items[FindIndex].Selected = true;
                }
                else
                {
                    MessageBox.Show("The item '" + Tools.FindItem + "' was not found.", "Find",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            Tools.ButtonPress = 0;
        }

        private void Button5_Click(object sender, EventArgs e)
        {
            //Show about form
            About frm = new About();
            frm.ShowDialog();
        }

        private void Button6_Click(object sender, EventArgs e)
        {
            DialogResult dr = MessageBox.Show("Do you want to exit now?", "Exit",
                MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation);
            if (dr == DialogResult.Yes)
            {
                //Close program
                this.Close();
            }
        }

        private void LstV_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar == '\r') & LstV.Items.Count>0)
            {
                e.Handled = true;
                Button2_Click(sender, e);
            }
        }

        private void Button7_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Title = "Open";
            ofd.Filter = "Key Files(*.key)|*.key";

            if (ofd.ShowDialog() == DialogResult.OK)
            {
                //Set the filename
                Filename = ofd.FileName;
                //Reload the keys
                //Disable buttons
                Button2.Enabled = false;
                Button3.Enabled = false;
                mnuEdit.Enabled = false;
                mnuDelete.Enabled = false;
                mnuCopy.Enabled = false;
                LoadKeys(0);
            }
        }

        private void Button8_Click(object sender, EventArgs e)
        {
            //Make backup of open file
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Title = "Backup";
            sfd.Filter = "Key Files(*.key)|*.key";

            if (sfd.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    //Do a copy of the filename
                    File.Copy(Filename, sfd.FileName, true);
                }
                catch (Exception Error)
                {
                    MessageBox.Show(Error.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void Button9_Click(object sender, EventArgs e)
        {
            SaveFileDialog fsd = new SaveFileDialog();
            fsd.Title = "Export To Text";
            fsd.Filter = "Text Files(*.txt)|*.txt";

            if (fsd.ShowDialog() == DialogResult.OK)
            {
                //Do export
                Export(fsd.FileName);
            }
        }

        private void LstV_KeyDown(object sender, KeyEventArgs e)
        {
            if ((e.KeyCode == Keys.Delete) & LstV.Items.Count>0)
            {
                Button3_Click(sender, e);
            }
        }

        private void button10_Click(object sender, EventArgs e)
        {

        }
    }
}
